﻿using System;
using System.Threading;		//Sleep
using UsbHid;
using UsbHid.IOCtrl;

namespace hidio1
{
	class Program
	{
		static void Main(string[] args)
		{
			Console.WriteLine("【AVR-HID/IO テストプログラム】ATmega48シリーズ版");
			
			HidDeviceMgr hidMgr = new HidDeviceMgr();
			HidDeviceInfo[] hidInfos = hidMgr.GetDeviceList(0x16C0, 0x05DF);	//[AVR-HID/IO]のVID,PID
			if (hidInfos.Length == 0)
			{
				Console.WriteLine("該当するデバイスが見つかりませんでした。");
				return;
			}
			
			//目的のデバイスが0番目だったとする
			//・確実に判断するには、ベンダー名、プロダクト名、シリアル番号などを照合する。
			HidDevice hid = hidMgr.TargetDevice(0);
			
			//【AVRの割り込み機能を利用する場合】
			//インタラプト転送(IN)のコールバック関数を登録し、受信を開始する
			hid.InterruptInStart(VectorNum);
			
			//[AVR-HID/IO]を操作するオブジェクト
			HidIOCtrl hidio = new HidIOCtrl(hid);

			//-------------------------------------------------------
			//LEDチカチカ
			//-------------------------------------------------------
			ConsoleKeyInfo keyInfo;
			Console.WriteLine("[SPACE]キーでLEDチカチカテスト開始。");
			while ((keyInfo = Console.ReadKey(true)).Key != ConsoleKey.Spacebar) { }
			
			hidio.Set(SFR.DDRC, 0x3F);	//PC5-PC0を出力方向に
			for (int i = 0; i < 3; i++)
			{
				hidio.Set(SFR.PORTC, 0x3F);	//Hiを出力
				Thread.Sleep(500);
				hidio.Set(SFR.PORTC, 0x00);	//Loを出力
				Thread.Sleep(500);
			}

			hidio.Set(SFR.PORTC, 0x3F);	//--111111
			//And
			Console.WriteLine("[SPACE]キーで[--110000]をANDします。->[--110000]");
			while ((keyInfo = Console.ReadKey(true)).Key != ConsoleKey.Spacebar) { }
			hidio.AndSet(SFR.PORTC, 0x30);	//-> --110000
			//Or
			Console.WriteLine("[SPACE]キーで[--111100]をOR します。->[--111100]");
			while ((keyInfo = Console.ReadKey(true)).Key != ConsoleKey.Spacebar) { }
			hidio.OrSet(SFR.PORTC, 0x3C);	//-> --111100
			//Xor
			Console.WriteLine("[SPACE]キーで[--101010]をXORします。->[--010110]");
			while ((keyInfo = Console.ReadKey(true)).Key != ConsoleKey.Spacebar) { }
			hidio.XorSet(SFR.PORTC, 0x2A);	//-> --010110

			//レジスタを読み取る例
			Console.WriteLine("[SPACE]キーで現在のPORTCの値を取得します。0x16のはずです。");
			while ((keyInfo = Console.ReadKey(true)).Key != ConsoleKey.Spacebar) { }
			int port = hidio.Get(SFR.PORTC);
			Console.WriteLine("PORTC = 0x{0:X2}", port);

			Console.WriteLine();
			Console.WriteLine("[SPACE]キーでテストを続行します。");
			while ((keyInfo = Console.ReadKey(true)).Key != ConsoleKey.Spacebar) { }
			
			//使ったポートの後始末
			hidio.Set(SFR.PORTC, 0x00);
			hidio.Set(SFR.DDRC, 0x00);

			//-------------------------------------------------------
			//EEPROM読み書きテスト
			//-------------------------------------------------------
			Console.WriteLine();
			Console.WriteLine("[SPACE]キーでEEPROMに 0x11, 0x2233 を書き込みます。");
			while ((keyInfo = Console.ReadKey(true)).Key != ConsoleKey.Spacebar) { }
			hidio.EEPRomWriteByte(0, 0x11);
			hidio.EEPRomWriteWord(4, 0x2233);	//特につながりのないアドレスに書き込んでみる
			
			Console.WriteLine("[SPACE]キーでEEPROMから値を読み込みます。");
			while ((keyInfo = Console.ReadKey(true)).Key != ConsoleKey.Spacebar) { }
			int eepByte = hidio.EEPRomReadByte(0);
			int eepWord = hidio.EEPRomReadWord(4);
			Console.WriteLine("0x{0:X2}, 0x{1:X4} が読み込めました。合っていますか？", eepByte, eepWord);

			Console.WriteLine();
			Console.WriteLine("[SPACE]キーでテストを続行します。");
			while ((keyInfo = Console.ReadKey(true)).Key != ConsoleKey.Spacebar) { }
			
			//-------------------------------------------------------
			//タイマーで割り込みを発生させる
			//-------------------------------------------------------
			Console.WriteLine();
			Console.WriteLine("[SPACE]キーで割り込みキャッチテスト開始。[ESC]キーでアプリを終了します。");
			Console.WriteLine("1秒間隔でTIMER1_COMPA_vect（ATmega48/88/168では12番）が発生します。");
			while ((keyInfo = Console.ReadKey(true)).Key != ConsoleKey.Spacebar) { }

			//TIMER1_COMPA_vectの設定
			/*	このコメント内はAVR上で実際に動作するソース（C言語）をコピペしたもの。
				これをそのまま[AVR-HID/IO]用に書き換えればよい。
			//16bitタイマーをセットする
			//CTC動作	(TCCR1B:WGM13, WGM12; TCCR1A:WGM11, WGM10) = 0100
			//分周		(TCCR1B:CS12, CS11, CS10) = 1024分周:101, 256分周:100, 64分周:011, 8分周:010, 分周なし:001
			TCCR1A = (0 << WGM11) | (0 << WGM10);
			TCCR1B = (0 << WGM13) | (1 << WGM12) | (1 << CS12) | (0 << CS11) | (1 << CS10);
			OCR1A = 11718UL - 1;	//1000ms = (12MHz/1024分周) = 11718カウント
			TIMSK1 = (1 << OCIE1A);	//比較一致割り込みを許可する
			*/
			hidio.Set(SFR.TCCR1A, (0 << RB.WGM11) | (0 << RB.WGM10));
			hidio.Set(SFR.TCCR1B, (0 << RB.WGM13) | (1 << RB.WGM12) | (1 << RB.CS12) | (0 << RB.CS11) | (1 << RB.CS10));
			const int timerCount = 11718 - 1;
			hidio.Set(SFR.OCR1AH, (timerCount >> 8) & 0xFF);	//16bitレジスタにはHiバイトから書くこと（読むときはLoバイトから）
			hidio.Set(SFR.OCR1AL, timerCount & 0xFF);
			hidio.Set(SFR.TIMSK1, 1 << RB.OCIE1A);

			//[ESC]キー入力待ち
			while ((keyInfo = Console.ReadKey(true)).Key != ConsoleKey.Escape) { }
			
			//後始末
			hidio.Set(SFR.TCCR1A, 0x00);
			hidio.Set(SFR.TCCR1B, 0x00);
			hidio.Set(SFR.OCR1AH, 0x00);
			hidio.Set(SFR.OCR1AL, 0x00);
			hidio.Set(SFR.TIMSK1, 0x00);

			//【AVRの割り込み機能を利用した場合】
			//アプリ終了前にインタラプト転送(IN)の受信を終了する
			hid.InterruptInStop();
		}

		//【インタラプト転送(IN)受信時のコールバック】
		//AVRで割り込みが発生するとベクター番号（AVRのデータシートを参照）が通知されてくる。
		static void VectorNum(byte[] data)
		{
			UInt32 vectorBits = HidIOCtrl.ToBitFlag(data);

			for (int i = 0; i < 32; i++, vectorBits >>= 1)
			{
				if ((vectorBits & 1) == 1)
					Console.WriteLine("ベクター番号{0}の割り込みが発生しました。", i);
			}
		}
	}
}
