﻿using System;
using System.Threading;		//Sleep
using UsbHid;
using UsbHid.IOCtrl;

namespace hidio1
{
	class Program
	{
		static void Main(string[] args)
		{
			Console.WriteLine("【AVR-HID/IO テストプログラム】ATtiny2313版");

			HidDeviceMgr hidMgr = new HidDeviceMgr();
			HidDeviceInfo[] hidInfos = hidMgr.GetDeviceList(0x16C0, 0x05DF);	//[AVR-HID/IO]のVID,PID
			if (hidInfos.Length == 0)
			{
				Console.WriteLine("該当するデバイスが見つかりませんでした。");
				return;
			}
			
			//目的のデバイスが0番目だったとする
			//・確実に判断するには、ベンダー名、プロダクト名、シリアル番号などを照合する。
			HidDevice hid = hidMgr.TargetDevice(0);
			
			//[AVR-HID/IO]を操作するオブジェクト
			HidIOCtrl hidio = new HidIOCtrl(hid);

			//-------------------------------------------------------
			//LEDチカチカ
			//-------------------------------------------------------
			ConsoleKeyInfo keyInfo;
			Console.WriteLine("[SPACE]キーでLEDチカチカテスト開始。");
			while ((keyInfo = Console.ReadKey(true)).Key != ConsoleKey.Spacebar) { }
			
			hidio.Set(SFR.DDRB, 0xFF);	//PB7-PB0を出力方向に
			for (int i = 0; i < 3; i++)
			{
				hidio.Set(SFR.PORTB, 0xFF);	//Hiを出力
				Thread.Sleep(500);
				hidio.Set(SFR.PORTB, 0x00);	//Loを出力
				Thread.Sleep(500);
			}

			hidio.Set(SFR.PORTB, 0xFF);	//11111111
			//And
			Console.WriteLine("[SPACE]キーで[11110000]をANDします。->[11110000]");
			while ((keyInfo = Console.ReadKey(true)).Key != ConsoleKey.Spacebar) { }
			hidio.AndSet(SFR.PORTB, 0xF0);
			//Or
			Console.WriteLine("[SPACE]キーで[00111100]をOR します。->[11111100]");
			while ((keyInfo = Console.ReadKey(true)).Key != ConsoleKey.Spacebar) { }
			hidio.OrSet(SFR.PORTB, 0x3C);
			//Xor
			Console.WriteLine("[SPACE]キーで[10101010]をXORします。->[01010110]");
			while ((keyInfo = Console.ReadKey(true)).Key != ConsoleKey.Spacebar) { }
			hidio.XorSet(SFR.PORTB, 0xAA);

			//レジスタを読み取る例
			Console.WriteLine("[SPACE]キーで現在のPORTBの値を取得します。0x56のはずです。");
			while ((keyInfo = Console.ReadKey(true)).Key != ConsoleKey.Spacebar) { }
			int port = hidio.Get(SFR.PORTB);
			Console.WriteLine("PORTB = 0x{0:X2}", port);

			Console.WriteLine();
			Console.WriteLine("[SPACE]キーでテストを続行します。");
			while ((keyInfo = Console.ReadKey(true)).Key != ConsoleKey.Spacebar) { }
			
			//使ったポートの後始末
			hidio.Set(SFR.PORTB, 0x00);
			hidio.Set(SFR.DDRB, 0x00);

			//-------------------------------------------------------
			//EEPROM読み書きテスト
			//-------------------------------------------------------
			Console.WriteLine();
			Console.WriteLine("[SPACE]キーでEEPROMに 0x11, 0x22 を書き込みます。");
			while ((keyInfo = Console.ReadKey(true)).Key != ConsoleKey.Spacebar) { }
			hidio.EEPRomWriteByte(0, 0x11);
			hidio.EEPRomWriteByte(4, 0x22);		//特につながりのないアドレスに書き込んでみる
			
			Console.WriteLine("[SPACE]キーでEEPROMから値を読み込みます。");
			while ((keyInfo = Console.ReadKey(true)).Key != ConsoleKey.Spacebar) { }
			int eepByte1 = hidio.EEPRomReadByte(0);
			int eepByte2 = hidio.EEPRomReadByte(4);
			Console.WriteLine("0x{0:X2}, 0x{1:X2} が読み込めました。合っていますか？", eepByte1, eepByte2);

			Console.WriteLine();
			Console.WriteLine("[ESC]キーでアプリを終了します。");
			while ((keyInfo = Console.ReadKey(true)).Key != ConsoleKey.Escape) { }
			
		}	//end of main()

	}	//end of class

}	//end of namespace
